@@ -12,9 +12,10 @@ import json
 class GradescopeClient:
     base_url = "https://gradescope.com"
     login_path = "/login"
-    def __init__(self, logout_on_del: bool=False, logout_on_with: bool=False):
+    def __init__(self, logout_on_del: bool=False, logout_on_with: bool=False, user_agent: str="fullGSapi"):
         self.session = requests.Session()
-        headers={"User-Agent":"Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36"}
+        headers={"User-Agent":user_agent}
+        # headers={"User-Agent":"Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36"}
         self.session.headers.update(headers)
         self.logged_in = False
         self.last_res = None
@@ -63,19 +64,19 @@ class GradescopeClient:
         }
         if header_token is not None:
             headers["X-CSRF-Token"] = header_token
-        self.last_res = self.session.post(url, data=data, json=json, files=files, headers = headers)
-        return self.last_res
+        self.last_res = res = self.session.post(url, data=data, json=json, files=files, headers = headers)
+        return res
 
     def get_token(self, url, action=None, meta=None):
-        self.last_res = self.session.get(url)
-        self.last_soup = BeautifulSoup(self.last_res.content, "html.parser")
+        self.last_res = res = self.session.get(url)
+        self.last_soup = soup = BeautifulSoup(res.content, "html.parser")
         form = None
         if action:
-            form = self.last_soup.find("form", {"action":action})
+            form = soup.find("form", {"action":action})
         elif meta:
-            return self.last_soup.find("meta", {"name": meta})['content']
+            return soup.find("meta", {"name": meta})['content']
         else:
-            form = self.last_soup.find("form")
+            form = soup.find("form")
         return form.find("input", {"name":"authenticity_token"})['value']
 
     def log_in(self, email, password):
@@ -90,8 +91,8 @@ class GradescopeClient:
             "commit": "Log In",
             "session[remember_me_sso]": 0,
         }
-        self.last_res = self.submit_form(url, url, data=payload)
-        if self.last_res.ok:
+        self.last_res = res = self.submit_form(url, url, data=payload)
+        if res.ok:
             self.logged_in = True
             return True
         return False
@@ -113,8 +114,8 @@ class GradescopeClient:
         base_url = f"https://www.gradescope.com"
         url = base_url + "/logout"
         ref_url = base_url + "/account"
-        self.last_res = self.session.get(url, headers={"Referer": ref_url})
-        if self.last_res.ok:
+        self.last_res = res = self.session.get(url, headers={"Referer": ref_url})
+        if res.ok:
             self.logged_in = False
             return True
         return False
@@ -123,11 +124,11 @@ class GradescopeClient:
         if not self.logged_in:
             print("You must be logged in to download grades!")
             return False
-        self.last_res = self.session.get(f"https://www.gradescope.com/courses/{class_id}/assignments/{assignment_id}/scores.{filetype}")
-        if not self.last_res or not self.last_res.ok:
-            print(f"Failed to get a response from gradescope! Got: {self.last_res}")
+        self.last_res = res = self.session.get(f"https://www.gradescope.com/courses/{class_id}/assignments/{assignment_id}/scores.{filetype}")
+        if not res or not res.ok:
+            print(f"Failed to get a response from gradescope! Got: {res}")
             return False
-        return self.last_res.content
+        return res.content
     
     def regrade_submission(self, class_id: str, assignment_id: str, submission_id: str) -> bool:
         if not self.logged_in:
@@ -141,8 +142,8 @@ class GradescopeClient:
         payload = {
             "authenticity_token": token
         }
-        self.last_res = self.submit_form(regrade_url, url, data=payload)
-        return self.last_res.ok
+        self.last_res = res = self.submit_form(regrade_url, url, data=payload)
+        return res.ok
 
     def regrade_all(self, class_id: str, assignment_id: str) -> bool:
         if not self.logged_in:
@@ -155,8 +156,8 @@ class GradescopeClient:
         payload = {
             "authenticity_token": token
         }
-        self.last_res = self.submit_form(url, location_url, data=payload)
-        return self.last_res.ok
+        self.last_res = res = self.submit_form(url, location_url, data=payload)
+        return res.ok
 
     def rebuild_autograder(self, class_id: str, assignment_id: str, file_name: str) -> bool:
         if not self.logged_in:
@@ -180,8 +181,8 @@ class GradescopeClient:
         files = {
             "autograder_zip": (file_name, open(file_name, 'rb'))
         }
-        self.last_res = self.submit_form(url, referer_url, data=payload, files=files)
-        return self.last_res.ok
+        self.last_res = res = self.submit_form(url, referer_url, data=payload, files=files)
+        return res.ok
 
     def set_manual_ag_config(self, class_id: str, assignment_id: str, image: str) -> bool:
         if not self.logged_in:
@@ -203,8 +204,8 @@ class GradescopeClient:
         files = {
             "autograder_zip": ("", "")
         }
-        self.last_res = self.submit_form(url, referer_url, data=payload, files=files)
-        return self.last_res.ok
+        self.last_res = res = self.submit_form(url, referer_url, data=payload, files=files)
+        return res.ok
         
     def ag_building_data(self, class_id: str, assignment_id: str):
         if not self.logged_in:
@@ -214,12 +215,12 @@ class GradescopeClient:
         location_url = f"/courses/{class_id}/assignments/{assignment_id}"
         referer_url = base_url + location_url + "/configure_autograder"
         url = base_url + location_url
-        self.last_res = self.session.get(referer_url)
-        self.last_soup = BeautifulSoup(self.last_res.content, "html.parser")
+        self.last_res = res = self.session.get(referer_url)
+        self.last_soup = soup = BeautifulSoup(res.content, "html.parser")
         # cdata = self.last_soup.find(text=re.compile("CDATA"))
         # matches = re.search(r"\"status\"\s*:\s*\"(.*?)\"", cdata)
         # return matches[1]
-        cdata = GS_CDATA_decoder(soup=self.last_soup)
+        cdata = GS_CDATA_decoder(soup=soup)
         return cdata.get_gon()
 
     def get_submission_data(self, class_id: str, assignment_id: str, submission_id: str):
@@ -229,9 +230,9 @@ class GradescopeClient:
         base_url = f"https://www.gradescope.com"
         location_url = f"/courses/{class_id}/assignments/{assignment_id}/submissions/{submission_id}"
         url = base_url + location_url
-        self.last_res = self.session.get(url)
-        if self.last_res.ok:
-            return self.last_res.content
+        self.last_res = res = self.session.get(url)
+        if res.ok:
+            return res.content
     
     def get_autograder(self, class_id: str, assignment_id: str):
         if not self.logged_in:
@@ -246,9 +247,9 @@ class GradescopeClient:
         base_url = f"https://www.gradescope.com"
         location_url = f"/courses/{class_id}/assignments/{assignment_id}/docker_images/{docker_image_id}.json"
         url = base_url + location_url
-        self.last_res = self.session.get(url)
-        if self.last_res.ok:
-            return json.loads(self.last_res.content)
+        self.last_res = res = self.session.get(url)
+        if res.ok:
+            return json.loads(res.content)
         return {}
 
     def update_online_assignment(self, class_id: str, assignment_id: str, outline: str) -> bool:
@@ -264,12 +265,12 @@ class GradescopeClient:
         base_url = f"https://www.gradescope.com"
         location_url = f"/courses/{class_id}/assignments/{assignment_id}/outline/edit"
         url = base_url + location_url
-        self.last_res = self.session.get(url)
-        if not self.last_res or not self.last_res.ok:
-            print(f"Failed to get a response from gradescope! Got: {self.last_res}")
+        self.last_res = res = self.session.get(url)
+        if not res or not res.ok:
+            print(f"Failed to get a response from gradescope! Got: {res}")
             return ""
-        self.last_soup = BeautifulSoup(self.last_res.content, "html.parser")
-        editors = self.last_soup.find_all("div", {"data-react-class": "AssignmentEditor"})
+        self.last_soup = soup = BeautifulSoup(res.content, "html.parser")
+        editors = soup.find_all("div", {"data-react-class": "AssignmentEditor"})
         if len(editors) == 0:
             print(f"Could not find online submission data!")
             return ""
@@ -282,12 +283,12 @@ class GradescopeClient:
         base_url = f"https://www.gradescope.com"
         location_url = f"/courses/{class_id}/assignments/{assignment_id}/submissions/new"
         url = base_url + location_url
-        self.last_res = self.session.get(url)
-        if not self.last_res or not self.last_res.ok:
-            print(f"Failed to get a response from gradescope! Got: {self.last_res}")
+        self.last_res = res = self.session.get(url)
+        if not res or not res.ok:
+            print(f"Failed to get a response from gradescope! Got: {res}")
             return ""
-        self.last_soup = BeautifulSoup(self.last_res.content, "html.parser")
-        editors = self.last_soup.find_all("div", {"data-react-class": "OnlineAssignmentSubmitter"})
+        self.last_soup = soup = BeautifulSoup(res.content, "html.parser")
+        editors = soup.find_all("div", {"data-react-class": "OnlineAssignmentSubmitter"})
         if len(editors) == 0:
             print(f"Could not find online submission data!")
             return ""
@@ -318,12 +319,12 @@ class GradescopeClient:
         base_url = f"https://www.gradescope.com"
         location_url = f"/courses/{class_id}/assignments/{assignment_id}/outline/edit"
         url = base_url + location_url
-        self.last_res = self.session.get(url)
-        if not self.last_res or not self.last_res.ok:
-            print(f"Failed to get a response from gradescope! Got: {self.last_res}")
+        self.last_res = res = self.session.get(url)
+        if not res or not res.ok:
+            print(f"Failed to get a response from gradescope! Got: {res}")
             return ""
-        self.last_soup = BeautifulSoup(self.last_res.content, "html.parser")
-        editors = self.last_soup.find_all("div", {"data-react-class": "AssignmentOutline"})
+        self.last_soup = soup = BeautifulSoup(res.content, "html.parser")
+        editors = soup.find_all("div", {"data-react-class": "AssignmentOutline"})
         if len(editors) == 0:
             print(f"Could not find online submission data!")
             return ""
@@ -348,9 +349,9 @@ class GradescopeClient:
                 "weight": str(weight),
                 }
         }
-        self.last_res = self.session.post(ref_url, headers=headers, json=payload)
-        if self.last_res.ok:
-            return self.last_res.content
+        self.last_res = res = self.session.post(ref_url, headers=headers, json=payload)
+        if res.ok:
+            return res.content
     
     def update_rubric_item(self, class_id: str, question_id: str, item_id: str, description: str=None, weight: float=None) -> dict:
         if not self.logged_in:
@@ -375,9 +376,9 @@ class GradescopeClient:
             payload["description"] = description
         if weight is not None:
             payload["weight"] = weight
-        self.last_res = self.session.put(url, headers=headers, json=payload)
-        if self.last_res.ok:
-            return self.last_res.content
+        self.last_res = res = self.session.put(url, headers=headers, json=payload)
+        if res.ok:
+            return res.content
 
     def delete_rubric_item(self, class_id: str, question_id: str, item_id: str) -> bool:
         if not self.logged_in:
@@ -393,8 +394,8 @@ class GradescopeClient:
             "Referer": ref_url,
             "X-CSRF-Token": self.get_token(ref_url, meta="csrf-token")
         }
-        self.last_res = self.session.delete(url, headers=headers)
-        return self.last_res.ok
+        self.last_res = res = self.session.delete(url, headers=headers)
+        return res.ok
 
     def get_grade_submission_data(self, class_id: str, question_id: str, submission_id: str) -> dict:
         if not self.logged_in:
@@ -410,9 +411,9 @@ class GradescopeClient:
             "Referer": ref_url,
             "X-CSRF-Token": self.get_token(ref_url, meta="csrf-token")
         }
-        self.last_res = self.session.get(url, headers=headers)
-        if self.last_res.ok:
-            return json.loads(self.last_res.content)
+        self.last_res = res = self.session.get(url, headers=headers)
+        if res.ok:
+            return json.loads(res.content)
 
     def update_rubric_items(self, class_id: str, question_id: str, data: dict) -> bool:
         """ (can modify position, weight and description)
@@ -431,8 +432,8 @@ class GradescopeClient:
             "Referer": ref_url,
             "X-CSRF-Token": self.get_token(ref_url, meta="csrf-token")
         }
-        self.last_res = self.session.patch(url, headers=headers, json=data)
-        return self.last_res.ok
+        self.last_res = res = self.session.patch(url, headers=headers, json=data)
+        return res.ok
 
     def grading_save(self, class_id: str, question_id: str, submission_id: str, data: dict, save_group: bool=False):
         """
@@ -452,8 +453,8 @@ class GradescopeClient:
             "Referer": ref_url,
             "X-CSRF-Token": self.get_token(ref_url, meta="csrf-token")
         }
-        self.last_res = self.session.post(url, headers=headers, json=data)
-        return self.last_res.ok
+        self.last_res = res = self.session.post(url, headers=headers, json=data)
+        return res.ok
 
     def publish_grades(self, class_id: str, assignment_id: str, publish: bool=True) -> bool:
         if not self.logged_in:
@@ -469,8 +470,8 @@ class GradescopeClient:
             "authenticity_token": token,
             "assignment[published]": publish
         }
-        self.last_res = self.submit_form(url, location_url, data=payload)
-        return self.last_res.ok
+        self.last_res = res = self.submit_form(url, location_url, data=payload)
+        return res.ok
 
     def edit_outline(self, class_id: str, assignment_id: str, data: dict) -> bool:
         """
@@ -565,8 +566,8 @@ class GradescopeClient:
             "Referer": ref_url,
             "X-CSRF-Token": self.get_token(ref_url, meta="csrf-token")
         }
-        self.last_res = self.session.patch(url, json=data, headers = headers)
-        return self.last_res
+        self.last_res = res = self.session.patch(url, json=data, headers = headers)
+        return res
 
     def grouping_set_answer_type(self, class_id: str, question_id: str, group_type: str) -> bool:
         """
@@ -595,8 +596,8 @@ class GradescopeClient:
                 "assisted_grading_type": group_type,
             },
         }
-        self.last_res = self.session.patch(url, json=data, headers = headers)
-        return self.last_res
+        self.last_res = res = self.session.patch(url, json=data, headers = headers)
+        return res
     
     def old_grouping_get_answer_groups(self, class_id: str, question_id: str) -> dict:
         if not self.logged_in:
@@ -605,12 +606,12 @@ class GradescopeClient:
         base_url = f"https://www.gradescope.com"
         location_url = f"/courses/{class_id}/questions/{question_id}/answer_groups"
         url = base_url + location_url
-        self.last_res = self.session.get(url)
-        if not self.last_res or not self.last_res.ok:
-            print(f"Failed to get a response from gradescope! Got: {self.last_res}")
+        self.last_res = res = self.session.get(url)
+        if not res or not res.ok:
+            print(f"Failed to get a response from gradescope! Got: {res}")
             return ""
-        self.last_soup = BeautifulSoup(self.last_res.content, "html.parser")
-        editors = self.last_soup.find_all("div", {"data-react-class": "AnswerGrouper"})
+        self.last_soup = soup = BeautifulSoup(res.content, "html.parser")
+        editors = soup.find_all("div", {"data-react-class": "AnswerGrouper"})
         if len(editors) == 0:
             print(f"Could not find data!")
             return ""
@@ -623,11 +624,11 @@ class GradescopeClient:
         base_url = f"https://www.gradescope.com"
         location_url = f"/courses/{class_id}/questions/{question_id}/answer_groups.json"
         url = base_url + location_url
-        self.last_res = self.session.get(url)
-        if not self.last_res or not self.last_res.ok:
-            print(f"Failed to get a response from gradescope! Got: {self.last_res}")
+        self.last_res = res = self.session.get(url)
+        if not res or not res.ok:
+            print(f"Failed to get a response from gradescope! Got: {res}")
             return ""
-        return json.loads(self.last_res.content)
+        return json.loads(res.content)
 
     def grading_create_group(self, class_id: str, question_id: str, title: str, internal_title: str=None) -> bool:
         if not self.logged_in:
@@ -642,9 +643,9 @@ class GradescopeClient:
             "internal_title": internal_title,
             "title": title,
         }
-        self.last_res = self.submit_form(url, location_url, json=payload, header_token=token)
-        if self.last_res.ok:
-            return json.loads(self.last_res.content)
+        self.last_res = res = self.submit_form(url, location_url, json=payload, header_token=token)
+        if res.ok:
+            return json.loads(res.content)
 
     def grading_add_to_group(self, class_id: str, question_id: str, group_id: str, submissions_ids: [int]) -> bool:
         """
@@ -663,20 +664,20 @@ class GradescopeClient:
             "answer_group_id": group_id,
             "submission_ids": submissions_ids,
         }
-        self.last_res = self.submit_form(url, location_url, json=payload, header_token=token)
+        self.last_res = res = self.submit_form(url, location_url, json=payload, header_token=token)
         # if self.last_res.ok:
         #     return json.loads(self.last_res.content)
-        return self.last_res.ok
+        return res.ok
 
     def export_evaluations(self, class_id: str, assignment_id: str) -> bytes:
         if not self.logged_in:
             print("You must be logged in to download grades!")
             return False
-        self.last_res = self.session.get(f"https://www.gradescope.com/courses/{class_id}/assignments/{assignment_id}/export_evaluations")
-        if not self.last_res or not self.last_res.ok:
-            print(f"Failed to get a response from gradescope! Got: {self.last_res}")
+        self.last_res = res = self.session.get(f"https://www.gradescope.com/courses/{class_id}/assignments/{assignment_id}/export_evaluations")
+        if not res or not res.ok:
+            print(f"Failed to get a response from gradescope! Got: {res}")
             return False
-        return self.last_res.content
+        return res.content
     
     def get_assignment_grader(self, class_id: str, assignment_id: str) -> GS_assignment_Grader:
         return GS_assignment_Grader(self, class_id, assignment_id)
@@ -688,12 +689,12 @@ class GradescopeClient:
         base_url = f"https://www.gradescope.com"
         location_url = f"/courses/{class_id}/questions/{question_id}/submissions/{submission_id}/grade"
         url = base_url + location_url
-        self.last_res = self.session.get(url)
-        if not self.last_res or not self.last_res.ok:
-            print(f"Failed to get a response from gradescope! Got: {self.last_res}")
+        self.last_res = res = self.session.get(url)
+        if not res or not res.ok:
+            print(f"Failed to get a response from gradescope! Got: {res}")
             return ""
-        self.last_soup = BeautifulSoup(self.last_res.content, "html.parser")
-        editors = self.last_soup.find_all("div", {"data-react-class": "SubmissionGrader"})
+        self.last_soup = soup = BeautifulSoup(res.content, "html.parser")
+        editors = soup.find_all("div", {"data-react-class": "SubmissionGrader"})
         if len(editors) == 0:
             print(f"Could not find online submission data!")
             return ""
@@ -706,11 +707,11 @@ class GradescopeClient:
         base_url = f"https://www.gradescope.com"
         location_url = f"/courses/{class_id}/questions/{question_id}/submissions/{submission_id}/grade.json"
         url = base_url + location_url
-        self.last_res = self.session.get(url)
-        if not self.last_res or not self.last_res.ok:
-            print(f"Failed to get a response from gradescope! Got: {self.last_res}")
+        self.last_res = res = self.session.get(url)
+        if not res or not res.ok:
+            print(f"Failed to get a response from gradescope! Got: {res}")
             return ""
-        return self.last_res.content
+        return res.content
 
     def grading_grade_first_ungraded_or_first(self, class_id: str, question_id: str) -> str:
         """
@@ -722,11 +723,11 @@ class GradescopeClient:
         base_url = f"https://www.gradescope.com"
         location_url = f"/courses/{class_id}/questions/{question_id}/submissions/grade_first_ungraded_or_first"
         url = base_url + location_url
-        self.last_res = self.session.get(url, headers={"referer": f"https://www.gradescope.com/courses/{class_id}/questions/{question_id}/answer_groups/ungrouped"})
-        if not self.last_res or not self.last_res.ok:
-            print(f"Failed to get a response from gradescope! Got: {self.last_res}")
+        self.last_res = res = self.session.get(url, headers={"referer": f"https://www.gradescope.com/courses/{class_id}/questions/{question_id}/answer_groups/ungrouped"})
+        if not res or not res.ok:
+            print(f"Failed to get a response from gradescope! Got: {res}")
             return
-        red_url = self.last_res.url
+        red_url = res.url
         reg_pat = r"https:\/\/www\.gradescope\.com\/courses\/[0-9]+\/questions\/[0-9]+\/submissions\/([0-9]+)\/grade"
         matches = re.match(reg_pat, red_url)
         if matches:
@@ -739,9 +740,9 @@ class GradescopeClient:
         base_url = f"https://www.gradescope.com"
         location_url = f"/courses/{class_id}"
         url = base_url + location_url + "/assignments/rubrics"
-        self.last_res = self.session.get(url)
-        if self.last_res.ok:
-            return json.loads(self.last_res.content)
+        self.last_res = res = self.session.get(url)
+        if res.ok:
+            return json.loads(res.content)
 
     def grouping_delete_answer_group(self, class_id: str, question_id: str, answer_group_id: str) -> bool:
         if not self.logged_in:
@@ -757,8 +758,8 @@ class GradescopeClient:
             "Referer": ref_url,
             "X-CSRF-Token": self.get_token(ref_url, meta="csrf-token")
         }
-        self.last_res = self.session.delete(url, headers=headers)
-        return self.last_res.ok
+        self.last_res = res = self.session.delete(url, headers=headers)
+        return res.ok
 
     def create_exam(self, class_id: str, title: str, template: str) -> str:
         """
@@ -787,15 +788,83 @@ class GradescopeClient:
         files = {
             "template_pdf": (template, open(template, 'rb'), "application/pdf")
         }
-        self.last_res = self.submit_form(url, url, data=payload, files=files)
-        if self.last_res.ok:
+        self.last_res = res = self.submit_form(url, url, data=payload, files=files)
+        if res.ok:
             template_url = ""
-            red_url = self.last_res.url
+            red_url = res.url
             reg_pat = r"https:\/\/www\.gradescope\.com\/courses/[0-9]+/assignments/([0-9]+)/outline/edit"
             matches = re.match(reg_pat, red_url)
             if matches:
                 return matches[1]
 
+    def create_programming_assignment(self, class_id: str, title: str, total_points: float, manual_grading: bool, student_submission: bool, release_date: str, due_date: str, allow_late_submissions: bool, group_submission: bool, leaderboard_enabled: bool, hard_due_date: str=None, group_size: int=None, leaderboard_max_entries: int=None) -> str:
+        """
+        Returns the assignment id
+
+        authenticity_token: 5tk+AedAm6Yl++q926aaxetR5WcoSgYUNGLpqhTzESHB49XHxWmirEafcZdsXgl9H6pe7Ryk25auktrSrdjZsA==
+        assignment[type]: ProgrammingAssignment
+        assignment[title]: New Assignment
+        assignment[total_points]: 23
+        assignment[manual_grading]: 0
+        assignment[student_submission]: true
+        assignment[release_date_string]: Oct 13 2020 07:00 PM
+        assignment[due_date_string]: Oct 17 2020 07:00 PM
+        assignment[allow_late_submissions]: 0
+        assignment[group_submission]: 0
+        assignment[leaderboard_enabled]: 0
+        ----
+        authenticity_token: uphwsAOixw1dd5gKuO+oA7tFWSxMAETPgTlS7tqAdPmdopt2IYv+Bz4TAyAPFzu7T77ipnjumU0byWGWY6u8aA==
+        assignment[type]: ProgrammingAssignment
+        assignment[title]: New 2 Assignment
+        assignment[total_points]: 100.1
+        assignment[manual_grading]: 0
+        assignment[manual_grading]: 1
+        assignment[student_submission]: true
+        assignment[release_date_string]: Oct 13 2020 07:00 PM
+        assignment[due_date_string]: Oct 15 2020 07:00 PM
+        assignment[allow_late_submissions]: 0
+        assignment[allow_late_submissions]: 1
+        assignment[hard_due_date_string]: Oct 16 2020 07:00 PM
+        assignment[group_submission]: 0
+        assignment[group_submission]: 1
+        assignment[group_size]: 
+        assignment[leaderboard_enabled]: 0
+        assignment[leaderboard_enabled]: 1
+
+        """
+        if not self.logged_in:
+            print("You must be logged in!")
+            return
+        url = self.base_url + f"/courses/{class_id}/assignments"
+        token = self.get_token(url, meta="csrf-token")
+        payload = {
+            "authenticity_token": token,
+            "assignment[type]": "ProgrammingAssignment",
+            "assignment[title]": title,
+            "assignment[total_points]": total_points,
+            "assignment[manual_grading]": int(manual_grading),
+            "assignment[student_submission]": student_submission,
+            "assignment[release_date_string]": release_date,
+            "assignment[due_date_string]": due_date,
+            "assignment[allow_late_submissions]": int(allow_late_submissions),
+            "assignment[group_submission]": int(group_submission),
+            "assignment[leaderboard_enabled]": int(leaderboard_enabled)
+        }
+        if allow_late_submissions:
+            payload["assignment[hard_due_date_string]"] = hard_due_date
+        if group_submission:
+            payload["assignment[group_size]"] = group_size
+        if leaderboard_enabled:
+            payload["assignment[leaderboard_max_entries]"] = leaderboard_max_entries
+
+        self.last_res = res = self.submit_form(url, url, data=payload)
+        if res.ok:
+            red_url = res.url
+            reg_pat = r"https:\/\/www\.gradescope\.com\/courses\/[0-9]+\/assignments\/([0-9]+)\/(?:configure_autograder|outline\/edit)"
+            matches = re.match(reg_pat, red_url)
+            if matches:
+                return matches[1]
+
     def scrape_autograder_results(self, class_id: str, assignment_id: str, submission_id: str):
         """
         Returns a dictionary of test case information:
@@ -820,8 +889,8 @@ class GradescopeClient:
         base_url = f"https://www.gradescope.com"
         location_url = f"/courses/{class_id}/assignments/{assignment_id}/submissions/{submission_id}"
         url = base_url + location_url
-        self.last_res = self.session.get(url)
-        self.last_soup = soup = BeautifulSoup(self.last_res.content, "html.parser")
+        self.last_res = res = self.session.get(url)
+        self.last_soup = soup = BeautifulSoup(res.content, "html.parser")
         header = soup.find_all("div", class_="testCase--header")
         body = soup.find_all("div", class_="testCase--body")
         tests = []
@@ -894,9 +963,9 @@ class GradescopeClient:
             return
         base_url = f"https://www.gradescope.com/courses/{class_id}/assignments/{assignment_id}/"
         url = base_url + f"review_grades"
-        self.last_res = self.session.get(url)
-        self.last_soup = BeautifulSoup(self.last_res.content, "html.parser")
-        cdata = GS_CDATA_decoder(soup=self.last_soup)
+        self.last_res = res = self.session.get(url)
+        self.last_soup = soup = BeautifulSoup(res.content, "html.parser")
+        cdata = GS_CDATA_decoder(soup=soup)
         return cdata.get_gon().get("generated_file")
     
     def check_export_submissions_progress(self, class_id: str, generated_files_id: str):
@@ -906,9 +975,9 @@ class GradescopeClient:
         base_url = f"https://www.gradescope.com"
         location_url = f"/courses/{class_id}"
         url = base_url + location_url + f"/generated_files/{generated_files_id}.json"
-        self.last_res = self.session.get(url)
-        if self.last_res.ok:
-            return json.loads(self.last_res.content)
+        self.last_res = res = self.session.get(url)
+        if res.ok:
+            return json.loads(res.content)
 
     def download_export_submissions(self, class_id: str, generated_files_id: str):
         if not self.logged_in:
@@ -917,16 +986,33 @@ class GradescopeClient:
         base_url = f"https://www.gradescope.com"
         location_url = f"/courses/{class_id}"
         url = base_url + location_url + f"/generated_files/{generated_files_id}.zip"
-        self.last_res = self.session.get(url)
-        if self.last_res.ok:
-            return self.last_res.content
+        self.last_res = res = self.session.get(url)
+        if res.ok:
+            return res.content
 
     def get_assignment_name(self, class_id: str, assignment_id: str) -> str:
+        """
+        Requires you to be a TA of the course.
+        """
         r = self.grading_get_rubrics(class_id)
         if r:
             for a in r.get("assignments", []):
                 if str(a.get("id")) == assignment_id:
                     return a.get("title")
+    
+    def get_assignment_id(self, class_id: str, assignment_name: str):
+        """
+        Requires you to be a TA of the course.
+        """
+        r = self.grading_get_rubrics(class_id)
+        if r:
+            for a in r.get("assignments", []):
+                if str(a.get("title")) == assignment_name:
+                    return a.get("id")
+
+    def student_get_assignment_id(self, class_id: str, assignment_name: str):
+        pass
+        #TODO IMPL ME
 
     def submit_via_repo(self, class_id: str, assignment_id: str, method: str, repo_identifier: str, branch: str, leaderboard_name: str="", owner_id: str=None):
         """
